#version 330

in vec2 TexCoord0;
in vec3 WorldPos0;
in vec3 Normal0;
in vec3 Tangent0;
out vec4 FragColor;

struct SpotLightStruct
{
  vec3 lightPos;
  float intensity;
  float factor;
  vec3 color;
};


uniform sampler2D textureSampler;
uniform sampler2D textureSamplerNormal;
uniform int spotLights_amount;
uniform SpotLightStruct spotLights[10];

uniform vec3 ambient_color;
uniform float ambient_intensity;

uniform vec3 diffuse_color;
uniform float diffuse_intensity;
uniform float diffuse_factor;
uniform vec3 diffuse_direction;

vec3 calcBumpedNormal()
{
    vec3 normal = normalize(Normal0);
    vec3 tangent = normalize(Tangent0);
    tangent = normalize(tangent - dot(tangent,normal) * normal);
    vec3 bitangent = cross(tangent, normal);
    vec3 bmapnormal = texture(textureSamplerNormal, TexCoord0.xy).xyz;
    bmapnormal = 2.0 * bmapnormal - vec3(1.0,1.0,1.0);
    vec3 bumpednormal;
    mat3 TBN = mat3(tangent,bitangent, normal);
    bumpednormal = TBN * bmapnormal;
    bumpednormal = normalize(bumpednormal);
    return bumpednormal;

}

void main()
{
 
    vec3 bumpnormal = calcBumpedNormal();
    //Ambient Term
    vec4 ambient = vec4(ambient_color,0) * ambient_intensity;
    //Diffuse Term:
    vec3 difdir = normalize(diffuse_direction);
    float diffuseFactor = dot(bumpnormal, -difdir);
    
    vec4 diffuse;
    if(diffuseFactor > 0)
    {
       diffuse = vec4(diffuse_color  * diffuseFactor * diffuse_intensity,0) ;
    }
    else
    {
       diffuse = vec4(0,0,0,0);
    }
    
//    vec4 pointLightsTotal = vec4(0,0,0,0);
//
//    for(int i = 0; i < spotLights_amount; i++)
//    {
//        float spotFactor = dot(bumpnormal, -normalize(WorldPos0 - spotLights[i].lightPos));
//        if(spotFactor > 0)
//        { 
//            pointLightsTotal += (vec4(spotLights[i].color,0) * spotLights[i].factor * spotLights[i].intensity * spotFactor)/length(WorldPos0 - spotLights[i].lightPos);
//        }
//    }

    
    //FragColor = texture2D(textureSampler, TexCoord0.xy) * (ambient + diffuse + pointLightsTotal);
    FragColor =  texture2D(textureSampler, TexCoord0.xy) * (diffuse + ambient_intensity);
    

}
