#version 330

in vec2 TexCoord0;
in vec3 WorldPos0;
in vec3 Normal0;


out vec4 FragColor;

struct SpotLightStruct
{
  vec3 lightPos;
  float intensity;
  float factor;
  vec3 color;
};

vec3 cube[9] = vec3[]( vec3(0.0,0.0,0.0),
                       vec3(1.0,-1.0,1.0),vec3(-1.0,-1.0,1.0),vec3(1.0,-1.0,-1.0),vec3(-1.0,-1.0,-1.0),
                       vec3(1.0,1.0,1.0),vec3(-1.0,1.0,1.0),vec3(1.0,1.0,-1.0),vec3(-1.0,1.0,-1.0) );

//vec3 cube[7] = vec3[]( vec3(0.0,0.0,0.0),
//                       vec3(1.0,0.0,0.0),vec3(0.0,0.0,1.0),vec3(-1.0,0.0,0.0),vec3(0.0,0.0,-1.0),vec3(0.0,1.0,0.0),vec3(0.0,-1.0,0.0) );
float pxw = 1.0/256.0 ;

uniform sampler2D textureSampler;
uniform int spotLights_amount;
uniform SpotLightStruct spotLights[10];
uniform int pcfLevel;

uniform vec3 ambient_color;
uniform float ambient_intensity;

uniform vec3 diffuse_color;
uniform float diffuse_intensity;
uniform float diffuse_factor;
uniform vec3 diffuse_direction;

uniform samplerCube cubemapTextureSampler;


void main()
{
    //Ambient Term
    vec4 ambient = vec4(ambient_color,0) * ambient_intensity;
    //Diffuse Term:
    float diffuseFactor = dot(normalize(Normal0), -diffuse_direction);
    
    vec4 diffuse;
    if(diffuseFactor > 0)
    {
       diffuse = vec4(diffuse_color,0) * diffuseFactor * diffuse_intensity;
    }
    else
    {
       diffuse = vec4(0,0,0,0);
    }
    
    vec4 pointLightsTotal = vec4(0,0,0,0);
    vec2 near_far = vec2(1,50); //ToDo: get near and far values!
    bool shadow = false;
    float shadowFactor;

    for(int i = 0; i < 1; i++)
    {
        float spotFactor = dot(normalize(Normal0), -normalize(WorldPos0 - spotLights[i].lightPos));
        if(spotFactor > 0)
        {  
            //vector from surface point to light position -> both in world space
            vec3 cubemap_lookup = spotLights[i].lightPos - WorldPos0;
            float curr_distance_to_light = length(cubemap_lookup);
            float pcf = 0;
            for(int f = 0; f < pcfLevel; f++)
            {
                vec3 cube_lookup_pcf = normalize(cubemap_lookup) + cube[f] * pxw;
                float cube_dist_light = texture(cubemapTextureSampler, cube_lookup_pcf).r * 50.0;
                if(cube_dist_light + 0.15 >=  curr_distance_to_light)
                {
                    pcf++;
                }

            }
            
            pcf/=9;
            pointLightsTotal += (vec4(spotLights[i].color,0) * spotLights[i].factor * spotLights[i].intensity * spotFactor*pcf)/length(WorldPos0 - spotLights[i].lightPos);
            

            

        }


    }

    FragColor = texture2D(textureSampler, TexCoord0.xy) * (ambient + diffuse + pointLightsTotal);
    
}
   


